@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
- * Copyright (C) 2021-2021 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -344,7 +344,7 @@ static DcmDataset* ParseDataset(const std::string& path)
   }
 
   // VL Whole Slide Microscopy Image IOD
-  OrthancWSI::DicomToolbox::SetStringTag(*dataset, DCM_SOPClassUID, "1.2.840.10008.5.1.4.1.1.77.1.6");
+  OrthancWSI::DicomToolbox::SetStringTag(*dataset, DCM_SOPClassUID, OrthancWSI::VL_WHOLE_SLIDE_MICROSCOPY_IMAGE_STORAGE_IOD);
 
   // Slide Microscopy
   OrthancWSI::DicomToolbox::SetStringTag(*dataset, DCM_Modality, "SM");  
@@ -643,10 +643,10 @@ static bool ParseParameters(int& exitStatus,
 
   boost::program_options::options_description volumeOptions("Description of the imaged volume");
   volumeOptions.add_options()
-    (OPTION_IMAGED_WIDTH, boost::program_options::value<float>()->default_value(15),
-     "Width of the specimen (in mm)")
-    (OPTION_IMAGED_HEIGHT, boost::program_options::value<float>()->default_value(15),
-     "Height of the specimen (in mm)")
+    (OPTION_IMAGED_WIDTH, boost::program_options::value<float>(),
+     "Width of the specimen (in mm), in the coordinate system of the glass slide, defaults to 15mm if missing")
+    (OPTION_IMAGED_HEIGHT, boost::program_options::value<float>(),
+     "Height of the specimen (in mm), in the coordinate system of the glass slide, defaults to 15mm if missing")
     (OPTION_IMAGED_DEPTH, boost::program_options::value<float>()->default_value(1),
      "Depth of the specimen (in mm)")
     (OPTION_OFFSET_X, boost::program_options::value<float>()->default_value(20), 
@@ -1010,6 +1010,7 @@ static bool ParseParameters(int& exitStatus,
 
 
 OrthancWSI::ITiledPyramid* OpenInputPyramid(OrthancWSI::ImageCompression& sourceCompression,
+                                            OrthancWSI::ImagedVolumeParameters& volume,
                                             const std::string& path,
                                             const OrthancWSI::DicomizerParameters& parameters)
 {
@@ -1072,9 +1073,28 @@ OrthancWSI::ITiledPyramid* OpenInputPyramid(OrthancWSI::ImageCompression& source
   {
     LOG(WARNING) << "Trying to open the input pyramid with OpenSlide";
     sourceCompression = OrthancWSI::ImageCompression_Unknown;
-    return new OrthancWSI::OpenSlidePyramid(path, 
-                                            parameters.GetTargetTileWidth(512), 
-                                            parameters.GetTargetTileHeight(512));
+
+    std::unique_ptr<OrthancWSI::OpenSlidePyramid> openslide(
+      new OrthancWSI::OpenSlidePyramid(path, parameters.GetTargetTileWidth(512),
+                                       parameters.GetTargetTileHeight(512)));
+
+    float volumeWidth, volumeHeight;
+    if (openslide->LookupImagedVolumeSize(volumeWidth, volumeHeight))
+    {
+      if (!volume.HasWidth())
+      {
+        volume.SetWidth(volumeWidth);
+        LOG(WARNING) << "Width of the imaged volume extracted using OpenSlide: " << volumeWidth << "mm";
+      }
+
+      if (!volume.HasHeight())
+      {
+        volume.SetHeight(volumeHeight);
+        LOG(WARNING) << "Height of the imaged volume extracted using OpenSlide: " << volumeHeight << "mm";
+      }
+    }
+
+    return openslide.release();
   }
   catch (Orthanc::OrthancException&)
   {
@@ -1101,12 +1121,22 @@ int main(int argc, char* argv[])
       OrthancWSI::ImageCompression sourceCompression;
       std::unique_ptr<OrthancWSI::ITiledPyramid> source;
 
-      source.reset(OpenInputPyramid(sourceCompression, parameters.GetInputFile(), parameters));
+      source.reset(OpenInputPyramid(sourceCompression, volume, parameters.GetInputFile(), parameters));
       if (source.get() == NULL)
       {
         throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
       }
 
+      // In the 2 lines below, remember to switch X/Y when going from physical to pixel coordinates!
+      float pixelSpacingX = volume.GetWidth() / static_cast<float>(source->GetLevelHeight(0));
+      float pixelSpacingY = volume.GetHeight() / static_cast<float>(source->GetLevelWidth(0));
+      if (std::abs(pixelSpacingX - pixelSpacingY) >= 100.0f * std::numeric_limits<float>::epsilon())
+      {
+        LOG(WARNING) << "Your pixel spacing is different along the X and Y axes, make sure that "
+                     << "you have not inversed the --" << OPTION_IMAGED_WIDTH << " and the --"
+                     << OPTION_IMAGED_HEIGHT << " options: " << pixelSpacingX << " vs. " << pixelSpacingY;
+      }
+
       LOG(WARNING) << "Compression of the individual source tiles: " << OrthancWSI::EnumerationToString(sourceCompression);
       
       // Create the shared DICOM tags
