@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
- * Copyright (C) 2021-2021 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -27,14 +27,43 @@
 
 namespace OrthancWSI
 {
-  ImagedVolumeParameters::ImagedVolumeParameters()
+  ImagedVolumeParameters::ImagedVolumeParameters() :
+    hasWidth_(false),
+    hasHeight_(false),
+    width_(0),
+    height_(0),
+
+    // Typical parameters for a specimen, in millimeters
+    depth_(1),
+    offsetX_(20),
+    offsetY_(40)
   {
-    // Typical parameters of a specimen millimeters
-    width_ = 15;
-    height_ = 15;
-    depth_ = 1;
-    offsetX_ = 20;
-    offsetY_ = 40;
+  }
+
+
+  float ImagedVolumeParameters::GetWidth() const
+  {
+    if (hasWidth_)
+    {
+      return width_;
+    }
+    else
+    {
+      return 15;  // Typical width of a specimen
+    }
+  }
+
+
+  float ImagedVolumeParameters::GetHeight() const
+  {
+    if (hasHeight_)
+    {
+      return height_;
+    }
+    else
+    {
+      return 15;  // Typical height of a specimen
+    }
   }
 
 
@@ -44,8 +73,11 @@ namespace OrthancWSI
     {
       throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
     }
-
-    width_ = width;
+    else
+    {
+      width_ = width;
+      hasWidth_ = true;
+    }
   }
     
 
@@ -55,8 +87,11 @@ namespace OrthancWSI
     {
       throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
     }
-
-    height_ = height;
+    else
+    {
+      height_ = height;
+      hasHeight_ = true;
+    }
   }
 
     
