@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
- * Copyright (C) 2021-2021 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -27,6 +27,8 @@ namespace OrthancWSI
   class ImagedVolumeParameters
   {
   private:
+    bool   hasWidth_;
+    bool   hasHeight_;
     float  width_;
     float  height_;
     float  depth_;
@@ -36,15 +38,19 @@ namespace OrthancWSI
   public:
     ImagedVolumeParameters();
 
-    float GetWidth() const
+    bool HasWidth() const
     {
-      return width_;
+      return hasWidth_;
+    }
+
+    bool HasHeight() const
+    {
+      return hasHeight_;
     }
     
-    float GetHeight() const
-    {
-      return height_;
-    }
+    float GetWidth() const;
+
+    float GetHeight() const;
     
     float GetDepth() const
     {
