@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
- * Copyright (C) 2021-2021 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -160,7 +160,7 @@ namespace OrthancWSI
     FullOrthancDataset dataset(orthanc, "/instances/" + instanceId + "/tags");
     DicomDatasetReader reader(dataset);
 
-    if (reader.GetMandatoryStringValue(Orthanc::DicomPath(Orthanc::DICOM_TAG_SOP_CLASS_UID)) != "1.2.840.10008.5.1.4.1.1.77.1.6" ||
+    if (reader.GetMandatoryStringValue(Orthanc::DicomPath(Orthanc::DICOM_TAG_SOP_CLASS_UID)) != VL_WHOLE_SLIDE_MICROSCOPY_IMAGE_STORAGE_IOD ||
         reader.GetMandatoryStringValue(Orthanc::DicomPath(Orthanc::DICOM_TAG_MODALITY)) != "SM")
     {
       throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
@@ -190,7 +190,7 @@ namespace OrthancWSI
         throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
       }
 
-      frames_.resize(countFrames);
+      frames_.reserve(countFrames);
 
       for (size_t i = 0; i < countFrames; i++)
       {
@@ -225,11 +225,11 @@ namespace OrthancWSI
         {
           LOG(ERROR) << "Frame " << i << " with unexpected tile location (" 
                      << x << "," << y << ") in instance: " << instanceId;
-          throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
         }
-
-        frames_[i].first = x / tileWidth_;
-        frames_[i].second = y / tileHeight_;
+        else
+        {
+          frames_.push_back(std::make_pair(x / tileWidth_, y / tileHeight_));
+        }
       }
     }
     else
