@@ -2,8 +2,8 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
- * Copyright (C) 2021-2021 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -26,6 +26,7 @@
 #include <Compatibility.h>  // For std::unique_ptr
 #include <Images/ImageProcessing.h>
 #include <OrthancException.h>
+#include <SerializationToolbox.h>
 #include <Logging.h>
 
 #include <memory>
@@ -50,4 +51,28 @@ namespace OrthancWSI
     tileHeight_(tileHeight)
   {
   }
+
+
+  bool OpenSlidePyramid::LookupImagedVolumeSize(float& width,
+                                                float& height) const
+  {
+    std::string s;
+    double mppx;
+    double mppy;
+
+    if (image_.LookupProperty(s, "openslide.mpp-x") &&
+        Orthanc::SerializationToolbox::ParseDouble(mppx, s) &&
+        image_.LookupProperty(s, "openslide.mpp-y") &&
+        Orthanc::SerializationToolbox::ParseDouble(mppy, s))
+    {
+      // In the 2 lines below, remember to switch X/Y when going from physical to pixel coordinates!
+      width = mppy / 1000.0 * static_cast<double>(image_.GetLevelHeight(0));
+      height = mppx / 1000.0 * static_cast<double>(image_.GetLevelWidth(0));
+      return true;
+    }
+    else
+    {
+      return false;
+    }
+  }
 }
